@@ -63,21 +63,21 @@ class C_Parser:
                                  dest='cov_output_dir',
                                  default='',
                                  type=str,
-                                 help='Target directory for coverity output files.If not specified, defaults to user_home/.synopsys/blackduck-c-cpp/output/project_name')
+                                 help='Target directory for coverity output files.If not specified, defaults to user_home/.blackduck/blackduck-c-cpp/output/project_name')
 
         self.parser.add_argument('-od', '--output_dir',
                                  metavar='output_dir',
                                  dest='output_dir',
                                  default='',
                                  type=str,
-                                 help='Target directory for blackduck-c-cpp output files.If not specified, defaults to user_home/.synopsys/blackduck-c-cpp/output/project_name')
+                                 help='Target directory for blackduck-c-cpp output files.If not specified, defaults to user_home/.blackduck/blackduck-c-cpp/output/project_name. output_dir should be outside of the build directory.')
 
         self.parser.add_argument('-s', '--skip_build',
                                  type=C_Parser.str2bool,
                                  nargs='?',
                                  const=True,
                                  default=False,
-                                 help='Skip build and use previously generated build data.')
+                                 help='Skip build and use previously generated build data. Make sure that your initial coverity wrapped build uses the --emit-link-units flag')
 
         self.parser.add_argument('-v', '--verbose',
                                  metavar='verbose',
@@ -112,7 +112,6 @@ class C_Parser:
 
         self.parser.add_argument('-a', '--api_token',
                                  dest='api_token',
-                                 required=True,
                                  metavar='api_token',
                                  type=str,
                                  help='Black Duck API token')
@@ -122,7 +121,7 @@ class C_Parser:
                                  dest='additional_sig_scan_args',
                                  type=str,
                                  default=None,
-                                 help='Any additional args to pass to the signature scanner')
+                                 help='Any additional args to pass to the signature scanner. IndividualFileMatching is by default turned on. To pass multiple params, you can pass it like additional_sig_scan_args: \'--snippet-matching --license-search\'. It accepts scan cli properties; Detect properties are not accepted here.')
 
         self.parser.add_argument('-i', '--insecure',
                                  dest='insecure',
@@ -142,13 +141,13 @@ class C_Parser:
                                  default=False,
                                  help='In case of GCP failure, force use of older version of Coverity (if present)')
 
-        self.parser.add_argument('-djs', '--disable_json_splitter',
-                                 dest='disable_json_splitter',
+        self.parser.add_argument('-djs', '--disable_bdio_json_splitter',
+                                 dest='disable_bdio_json_splitter',
                                  type=C_Parser.str2bool,
                                  nargs='?',
                                  const=True,
                                  default=False,
-                                 help='Disable the json splitter and always upload as a single scan')
+                                 help='Disable the json splitter and always upload as a single scan. For using json/bdio splitter, dryrun is needed, so please run in offline mode first.')
 
         self.parser.add_argument('-si', '--scan_interval',
                                  dest='scan_interval',
@@ -161,7 +160,21 @@ class C_Parser:
                                  default=4750000000,
                                  metavar='json_splitter_limit',
                                  type=int,
-                                 help='Set the limit for a scan size in bytes')
+                                 help='Set the limit for a scan size in bytes. For using json/bdio splitter, dryrun is needed, so please run in offline mode first.')
+
+        self.parser.add_argument('-bsfl', '--bdio_split_max_file_entries',
+                                 dest='bdio_split_max_file_entries',
+                                 default=100000,
+                                 metavar='bdio_split_max_file_entries',
+                                 type=int,
+                                 help='Set the limit for maximum scan node entries per generated BDIO file')
+
+        self.parser.add_argument('-bscn', '--bdio_split_max_chunk_nodes',
+                                 dest='bdio_split_max_chunk_nodes',
+                                 default=3000,
+                                 metavar='bdio_split_max_chunk_nodes',
+                                 type=int,
+                                 help='Set the limit for maximum scan node entries per single bdio-entry file')
 
         self.parser.add_argument('-dg', '--debug',
                                  metavar='debug',
@@ -197,7 +210,7 @@ class C_Parser:
                                  nargs='?',
                                  const=True,
                                  default=False,
-                                 help='Store bdba and sig tar files, sig scan json, and raw_bdio.csv to disk if offline mode is true')
+                                 help='Store bdba and sig zip files, sig scan json, and raw_bdio.csv to disk if offline mode is true. For scans over 5GB to use bdio/json splitter, please run in offline mode first. scan_cli_dir should be specified when run in offline mode to generate dryrun files.  Once the dryrun files are generated, use_offline_files: True can be set to upload those to hub.')
 
         self.parser.add_argument('-md', '--modes',
                                  metavar='modes',
@@ -218,7 +231,7 @@ class C_Parser:
                                  dest='scan_cli_dir',
                                  default=None,
                                  type=str,
-                                 help='Scan cli directory')
+                                 help='Scan cli directory. Ex: Providing scan_cli_dir as /home/../../Black_Duck_Scan_Installation/ instead of /home/../../Black_Duck_Scan_Installation/scan.cli-2022.4.0/ works.')
 
         self.parser.add_argument('-Cc', '--cov_configure_args',
                                  metavar='cov_configure_args',
@@ -226,7 +239,7 @@ class C_Parser:
                                  default={},
                                  type=yaml.safe_load,
                                  required=False,
-                                 help='Additional configuration commands to cov-configure for different compilers. Inputs taken are of format {"compiler":"compiler-type"}. There is a way to use coverity template configuration to reduce number of template compiler configurations with wildcards: example: "--compiler *g++ --comptype gcc" for adding x86_64-pc-linux-gnu-g++')
+                                 help='Additional configuration commands to cov-configure for different compilers. Inputs taken are of format {"compiler":"compiler-type"}. There is a way to use coverity template configuration to reduce number of template compiler configurations with wildcards: example: "--compiler *g++ --comptype gcc" for adding x86_64-pc-linux-gnu-g++ can be passed as cov_configure_args: {"*g++":"gcc"}')
 
         self.parser.add_argument('-ac', '--additional_coverity_params',
                                  metavar='additional_coverity_params',
@@ -240,7 +253,7 @@ class C_Parser:
                                  nargs='?',
                                  const=True,
                                  default=False,
-                                 help='Use expand_sig_files for creating exploded directory instead of tar in sig scanner mode')
+                                 help='Use expand_sig_files for creating exploded directory instead of zip in sig scanner mode')
 
         self.parser.add_argument('-po', '--port',
                                  type=int,
@@ -254,4 +267,29 @@ class C_Parser:
                                  default=False,
                                  help="Use if this is a bazel build - make sure you have followed the setup instructions for Coverity")
 
+        self.parser.add_argument('-pgn', '--project_group_name',
+                                 dest='project_group_name',
+                                 type=str,
+                                 help="This is same as --detect.project.group.name in detect. Sets the 'Project Group' to assign the project to. Must match exactly to an existing project group on Black Duck.")
+
+        self.parser.add_argument('-pgd', '--project_description',
+                                 dest='project_description',
+                                 type=str,
+                                 default=None,
+                                 help="This is same as --detect.project.description in detect. If project description is specified, your project will be created with this description.")
+
+        self.parser.add_argument('-scv', '--set_coverity_mode',
+                                 metavar='set_coverity_mode',
+                                 dest='set_coverity_mode',
+                                 type=str,
+                                 default=None,
+                                 help="specify coverity mode to 'cov-build' to force run with cov-build. cov-cli runs by default for coverity versions >= 2023.9 and cov-build for < 2023.9")
+
+        self.parser.add_argument('-fpc', '--force_pull_coverity_vers',
+                                 metavar='force_pull_coverity_vers',
+                                 dest='force_pull_coverity_vers',
+                                 type=str,
+                                 default=None,
+                                 help="For linux platforms, force pull 2022.9 or latest version of coverity if not auto downloaded by blackduck-c-cpp correctly by specifying -'old' or 'latest' respectively")
+
         self.args = self.parser.parse_args()
